@@ -141,7 +141,7 @@ class OneLogin_Saml2_Auth(object):
                 if not keep_local_session:
                     OneLogin_Saml2_Utils.delete_local_session(delete_session_cb)
 
-                in_response_to = OneLogin_Saml2_Logout_Request.get_id(OneLogin_Saml2_Utils.decode_base64_and_inflate(self.__request_data['get_data']['SAMLRequest']))
+                in_response_to = logout_request.id
                 response_builder = OneLogin_Saml2_Logout_Response(self.__settings)
                 response_builder.build(in_response_to)
                 logout_response = response_builder.get_response()
@@ -224,7 +224,7 @@ class OneLogin_Saml2_Auth(object):
 
     def get_last_error_reason(self):
         """
-        Return the reason for the last error
+        Returns the reason for the last error
 
         :returns: Reason of the last error
         :rtype: None | string
@@ -247,16 +247,22 @@ class OneLogin_Saml2_Auth(object):
             value = self.__attributes[name]
         return value
 
-    def login(self, return_to=None):
+    def login(self, return_to=None, force_authn=False, is_passive=False):
         """
         Initiates the SSO process.
 
         :param return_to: Optional argument. The target URL the user should be redirected to after login.
         :type return_to: string
 
+        :param force_authn: Optional argument. When true the AuthNReuqest will set the ForceAuthn='true'.
+        :type force_authn: string
+
+        :param is_passive: Optional argument. When true the AuthNReuqest will set the Ispassive='true'.
+        :type is_passive: string
+
         :returns: Redirection url
         """
-        authn_request = OneLogin_Saml2_Authn_Request(self.__settings)
+        authn_request = OneLogin_Saml2_Authn_Request(self.__settings, force_authn, is_passive)
 
         saml_request = authn_request.get_request()
         parameters = {'SAMLRequest': saml_request}
@@ -272,13 +278,19 @@ class OneLogin_Saml2_Auth(object):
             parameters['Signature'] = self.build_request_signature(saml_request, parameters['RelayState'])
         return self.redirect_to(self.get_sso_url(), parameters)
 
-    def logout(self, return_to=None):
+    def logout(self, return_to=None, name_id=None, session_index=None):
         """
         Initiates the SLO process.
 
         :param return_to: Optional argument. The target URL the user should be redirected to after logout.
         :type return_to: string
 
+        :param name_id: The NameID that will be set in the LogoutRequest.
+        :type name_id: string
+
+        :param session_index: SessionIndex that identifies the session of the user.
+        :type session_index: string
+
         :returns: Redirection url
         """
         slo_url = self.get_slo_url()
@@ -288,7 +300,10 @@ class OneLogin_Saml2_Auth(object):
                 OneLogin_Saml2_Error.SAML_SINGLE_LOGOUT_NOT_SUPPORTED
             )
 
-        logout_request = OneLogin_Saml2_Logout_Request(self.__settings)
+        if name_id is None and self.__nameid is not None:
+            name_id = self.__nameid
+
+        logout_request = OneLogin_Saml2_Logout_Request(self.__settings, name_id=name_id, session_index=session_index)
 
         saml_request = logout_request.get_request()
 
